@@ -2,7 +2,7 @@
 """
 Main function to setup the daemon process.
 
-Copyright (C) 2011, 2012 by Memset Ltd. http://www.memset.com/
+Copyright (C) 2011-2013 by Memset Ltd. http://www.memset.com/
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -36,7 +36,8 @@ import tempfile
 from Crypto import Random
 import paramiko
 from sftpcloudfs.server import CloudFilesSFTPServer
-from sftpcloudfs.constants import version, project_url, config_file
+from sftpcloudfs.constants import version, project_url, config_file, default_ks_service_type, \
+    default_ks_tenant_separator, default_ks_endpoint_type
 from ftpcloudfs.fs import CloudFilesFS
 
 class PIDFile(object):
@@ -89,12 +90,21 @@ class Main(object):
         global config_file
 
         # look for an alternative configuration file
-        try:
-            alt_config_file = sys.argv.index('--config')
-            alt_config_file = sys.argv[alt_config_file+1]
-            config_file = alt_config_file
-        except (ValueError, IndexError):
-            pass
+        alt_config_file = False
+        # used to show errors before we actually start parsing stuff
+        parser = OptionParser()
+        for arg in sys.argv:
+            if arg == '--config':
+                try:
+                    alt_config_file = sys.argv[sys.argv.index(arg)+1]
+                    config_file = alt_config_file
+                except IndexError:
+                    pass
+            elif arg.startswith('--config='):
+                _, alt_config_file = arg.split('=', 1)
+                if alt_config_file == '':
+                    parser.error("--config option requires an argument")
+                config_file = alt_config_file
 
         config = RawConfigParser({'auth-url': None,
                                   'host-key-file': None,
@@ -108,15 +118,24 @@ class Main(object):
                                   'pid-file': None,
                                   'uid': None,
                                   'gid': None,
+                                  # keystone auth 2.0 support
+                                  'keystone-auth': False,
+                                  'keystone-region-name': None,
+                                  'keystone-tenant-separator': default_ks_tenant_separator,
+                                  'keystone-service-type': default_ks_service_type,
+                                  'keystone-endpoint-type': default_ks_endpoint_type,
                                   })
 
-        config.read(config_file)
+        if not config.read(config_file) and alt_config_file:
+            # the default conf file is optional
+            parser.error("failed to read %s" % config_file)
+
         if not config.has_section('sftpcloudfs'):
             config.add_section('sftpcloudfs')
 
         parser = OptionParser(version="%prog " + version,
                               description="This is a SFTP interface to Rackspace " + \
-                                    "Cloud Files and Open Stack Object Storage (Swift).",
+                                    "Cloud Files and OpenStack Object Storage (Swift).",
                               epilog="Contact and support at: %s" % project_url)
 
         parser.add_option("-a", "--auth-url", dest="authurl",
@@ -184,6 +203,37 @@ class Main(object):
                           default=config.get('sftpcloudfs', 'gid'),
                           help="GID to drop the privileges to when in daemon mode")
 
+        parser.add_option('--keystone-auth',
+                          action="store_true",
+                          dest="keystone",
+                          default=config.get('sftpcloudfs', 'keystone-auth'),
+                          help="Use auth 2.0 (Keystone, requires keystoneclient)")
+
+        parser.add_option('--keystone-region-name',
+                          type="str",
+                          dest="region_name",
+                          default=config.get('sftpcloudfs', 'keystone-region-name'),
+                          help="Region name to be used in auth 2.0")
+
+        parser.add_option('--keystone-tenant-separator',
+                          type="str",
+                          dest="tenant_separator",
+                          default=config.get('sftpcloudfs', 'keystone-tenant-separator'),
+                          help="Character used to separate tenant_name/username in auth 2.0, " + \
+                              "default: TENANT%sUSERNAME" % default_ks_tenant_separator)
+
+        parser.add_option('--keystone-service-type',
+                          type="str",
+                          dest="service_type",
+                          default=config.get('sftpcloudfs', 'keystone-service-type'),
+                          help="Service type to be used in auth 2.0, default: %s" % default_ks_service_type)
+
+        parser.add_option('--keystone-endpoint-type',
+                          type="str",
+                          dest="endpoint_type",
+                          default=config.get('sftpcloudfs', 'keystone-endpoint-type'),
+                          help="Endpoint type to be used in auth 2.0, default: %s" % default_ks_endpoint_type)
+
         parser.add_option('--config',
                           type="str",
                           dest="config",
@@ -213,15 +263,17 @@ class Main(object):
                 CloudFilesFS(None, None)
             except (ValueError, TypeError):
                 parser.error("memcache: invalid server address, ip:port expected")
-        else:
-            # default is True, and there's no single worker operation
-            CloudFilesFS.single_cache = False
 
         self.pidfile = PIDFile(options.pid_file)
         if self.pidfile.is_locked():
             parser.error("pid-file found: %s\nIs the server already running?" % options.pid_file)
 
         options.max_children = config.get('sftpcloudfs', 'max-children')
+
+        if options.keystone:
+            keystone_keys = ('region_name', 'tenant_separator', 'service_type', 'endpoint_type')
+            options.keystone = dict((key, getattr(options, key)) for key in keystone_keys)
+
         self.options = options
 
     def setup_log(self):
@@ -259,7 +311,9 @@ class Main(object):
         server = CloudFilesSFTPServer((self.options.bind_address, self.options.port),
                                        host_key=self.host_key,
                                        authurl=self.options.authurl,
-                                       max_children=self.options.max_children)
+                                       max_children=self.options.max_children,
+                                       keystone=self.options.keystone,
+                                       )
 
         dc = daemon.DaemonContext()
         dc.pidfile = self.pidfile
